from threading import local

try:
    # Mixin for compatible middleware (to be refactored when all projects use Django 2):
    # https://docs.djangoproject.com/en/2.1/topics/http/middleware/#writing-your-own-middleware
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object

_user = local()


class CurrentUserMiddleware(MiddlewareMixin):
    """
    Middleware which stores request's user into global thread-safe variable.
    Must be introduced AFTER `django.contrib.auth.middleware.AuthenticationMiddleware`.
    """
    def process_request(self, request):
        _user.value = request.user

    def process_response(self, request, response):
        # this cleanup is required e.g. when running tests single-threaded
        try:
            del _user.value
        except AttributeError:
            pass
        return response

    @staticmethod
    def get_current_user():
        if hasattr(_user, 'value') and _user.value:
            return _user.value



from django.db.models import ForeignKey


def object_to_dict(obj, blacklisted_fields: list = None, include_id: bool = False) -> dict:
    """
    Returns a dict with all data defined in the model class as a key-value-dict
    Attention: Does not work for M2M fields!
    """

    # Default blacklist
    blacklisted_fields = blacklisted_fields if blacklisted_fields else []

    # Add default django primary key to blacklist
    if not include_id:
        blacklisted_fields.append('id')

    data = vars(obj)
    valid_data = {}

    valid_fields = []
    for f in obj.__class__._meta.get_fields():
        if type(f) != ForeignKey:
            valid_fields.append(f.name)
        else:
            valid_fields.append(f'{f.name}_id')

    for key, value in list(data.items()):
        if key in valid_fields and key not in blacklisted_fields:
            valid_data[key] = value

    return valid_data



import os
import re
import subprocess
import time

from django.conf import settings


def generate_video_thumbnail(videofile, fallback_icon_path, image_path):
    base_path = settings.BASE_PATH
    file_path_wo_extension, extension = os.path.splitext(videofile.name)
    video_path = videofile.url
    full_path = "%s%s" % (base_path, video_path)
    length = get_video_length(full_path)

    try:
        duration = time.strptime(length, "%H:%M:%S")
        hour = duration.tm_hour
        min = duration.tm_min
        sec = duration.tm_sec
        # CONVERT TO SECONDS

        total_seconds = ((hour * 60) + min) * 60 + sec
        middle_point = total_seconds // 2
        new_hor = middle_point // 3600
        new_minu = (middle_point - (new_hor * 3600)) // 60
        new_seg = middle_point - ((new_hor * 3600) + (new_minu * 60))
        middle_point_full_time = "%02d:%02d:%02d" % (new_hor, new_minu, new_seg)

    except ValueError:
        return fallback_icon_path

    try:
        command = ("ffmpeg -ss %s -i %s/%s -vframes 1 %s/%s -y" % (
            middle_point_full_time, base_path, video_path, base_path, image_path)).split()
        subprocess.call(command)
        return image_path

    except Exception:
        return fallback_icon_path


def get_video_length(filename):
    try:
        result = subprocess.Popen(["ffprobe", filename], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    except Exception:
        return "00:00:00"

    return_str = [x for x in result.stdout.readlines() if "Duration" in x]

    if len(return_str) > 0:
        matches = re.match(r'Duration:\s([0-9:]+)\.[0-9]+,', str(return_str[0]).strip())
        return matches.group(1)
    else:
        return "00:00:00"



from ai_django_core.utils import log_whodid


def test_log_who_did_new_object(mocker):
    """
    Tests if a new object that doesn't have a value for the ``created_by`` and ``lastmodified_by`` fields, is handled
    properly.
    """
    user = mocker.MagicMock()
    obj = mocker.MagicMock()
    obj.created_by = None
    log_whodid(obj, user)
    assert obj.created_by == user
    assert obj.lastmodified_by == user


def test_log_who_did_existing_values(mocker):
    """
    Tests if, for an object that already has a value for ``created_by``, only the ``lastmodified_by`` field is updated.
    """
    user = mocker.MagicMock()
    obj = mocker.MagicMock()
    old_user = obj.created_by
    log_whodid(obj, user)
    assert obj.created_by == old_user
    assert obj.lastmodified_by == user
